// EAllocatorPtr.h: interface for the EAllocatorPtr class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EALLOCATORPTR_H__CB0A9032_BC16_11D2_91F2_004005446EE6__INCLUDED_)
#define AFX_EALLOCATORPTR_H__CB0A9032_BC16_11D2_91F2_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/* History:
 * 29/10/1999: BugFix: copyToThis, bug when copying object, 
 *                     they where destructed !
 */


#include "EAllocator.h"

template< class TElm,class EParentAllocator = EAllocatorT< TElm > >
class EAllocatorPtrT : public EParentAllocator
{
public:
  /*! Default Constructor.
   * Set the ownership flag to \c true. (pointed object are destructed).
   */
  EAllocatorPtrT();

  /*! Destructor.
   */
  virtual ~EAllocatorPtrT() {}

  /*! Set the ownership flag.
   * @param bOwnObject If \a bOwnObject is \c true then the allocator owns the
   *                   object. This means they are destroyed when destruct() is called.
   *                   If \a bOwnObject is \c false then object are not destroyed.
   */
  virtual void setOwnership( bool bOwnObject = true );

  /*! Tests if the allocator owns the object.
   */
  virtual bool ownership() const;

  /*! Initializes an object of the array.
   *
   * @param nIndex Zero based index of the object to initialize.
   */
  virtual void construct( SL nIndex );

  /*! Initializes a sequence of object in the array.
   *
   * @param nIndex Zero based index of the first object to initialize.
   * @param nLen Number of object to initialize.
   */
  virtual void construct( SL nIndex,
                          SL nLen );

  /*! Destructs an object of the array.
   * @param nIndex Zero based index of the object to destruct.
   */
  virtual void destruct( SL nIndex );

  /*! Destructs a sequence list of object in the array.
   *
   * delete is apply to each non NULL pointer.
   *
   * @param nIndex Zero based index of the first object to destruct. 
   * @param nLen Number of object to destruct. May be 0.
   */
  virtual void destruct( SL nIndex, 
                         SL nLen );

  /*! Copy a sequence of object from this array to another one.
   * @param nIndexSource Zero based index of the first object of the sequence to copy.
   * @param nLen Number of object in the sequence. May be zero.
   * @param nIndexDestination Zero based index of destination array where the sequence
   *                          is copied to.
   * @param allocatorDest Destination array.
   */
  virtual void copyTo( SL nIndexSource, 
                       SL nLen,
                       SL nIndexDestination,
                       EAllocatorT<TElm> &allocatorDest ) const;

  /*! Copy a sequence of object from a standard C array to this array.
   * @param arraySource Pointer on the first element of the source array to copy.
   * @param nLen Number of objects in the sequence. May be zero.
   * @param nIndexDestination Zero based index where the sequence is copied to.
   */
  virtual void copyFrom( const TElm *arraySource, 
                         SL nLen,
                         SL nIndexDestination );

  /*! Move a sequence of object inside the array.
   * @param nIndex Zero based index of the first element to move.
   * @param nLen Number of elements to move.
   * @param nIndexDestination Zero based index where the first element is moved to.
   */
  virtual void copyToThis( SL nIndex,
                           SL nLen,
                           SL nIndexDestination );

  /*! Copy another allocator to this one.
   * @param allocator Allocator to copy.
   */
  virtual void copy( const EParentAllocator &allocator );

private:
  /**@shapeType DependencyLink*/
  /*#  EAllocatorT lnkUnnamed*/
  bool _bOwnership;
};



template<class TElm,class EParentAllocator>
inline
EAllocatorPtrT<TElm,EParentAllocator>::EAllocatorPtrT()
{
  _bOwnership = true;
}


template<class TElm,class EParentAllocator>
void 
EAllocatorPtrT<TElm,EParentAllocator>::copyTo( SL nIndexSource, 
                                               SL nLen,
                                               SL nIndexDestination,
                                               EAllocatorT<TElm> &allocatorDest ) const
{
  EASSERT( nIndexSource <= count() );
  EASSERT( nLen >= 0 );
  EASSERT( nIndexSource + nLen <= count() );
  EASSERT( nIndexDestination <= allocatorDest.count() );
  EASSERT( nIndexDestination + nLen <= allocatorDest.count() );

  if ( !nLen )
    return;

  allocatorDest.destruct( nIndexDestination,
                          nLen );

//  allocatorDest.construct( nIndexDestination,
//                           nLen );

  memcpy( &allocatorDest.at( nIndexDestination ), 
          &at( nIndexSource ),
          sizeof( TElm ) * nLen );
}


template<class TElm,class EParentAllocator>
void 
EAllocatorPtrT<TElm,EParentAllocator>::copyFrom( const TElm *arraySource, 
                                                 SL nLen,
                                                 SL nIndexDestination )
{
  EASSERT( nIndexDestination >= 0 );
  EASSERT( nLen >= 0 );
  EASSERT( nIndexDestination + nLen <= count() );

  if ( !nLen )
    return;

  EASSERT( arraySource != NULL );

  destruct( nIndexDestination,
            nLen );

  memcpy( &at( nIndexDestination ),
          arraySource,
          nLen * sizeof( TElm ) );
}


template<class TElm,class EParentAllocator>
void 
EAllocatorPtrT<TElm,EParentAllocator>::copyToThis( SL nIndex,
                                                   SL nLen,
                                                   SL nIndexDestination )
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex + nLen <= count() );
  EASSERT( nIndexDestination >= 0 );
  EASSERT( nIndexDestination + nLen <= count() );

  if ( nIndex == nIndexDestination  ||
       !nLen )
    return;

  if ( nIndexDestination + nLen < nIndex ||     // destination range does not
       nIndexDestination > nIndex + nLen )      // overlap source !
  {
    destruct( nIndexDestination, 
              nLen );

    memmove( &at( nIndexDestination ),
             &at( nIndex ),
             nLen * sizeof( TElm ) );
  }
  else
  {
    if ( nIndex > nIndexDestination )
    {
      TElm *pSrcElm = &at( nIndex );
      SL nIndexCur = nIndexDestination;
      destruct( nIndexDestination, nIndex - nIndexDestination );
      while ( nLen-- )
      {
        at( nIndexCur++ ) = *pSrcElm;
        *pSrcElm++ = NULL;
      }
    }
    else      // reverse order copy so we do not overwrite source
    {
      TElm *pSrcElm = &at( nIndex ) + nLen;
      SL nIndexCur = nIndexDestination + nLen;
      destruct( nIndexDestination, nIndex + nLen - nIndexDestination );
      while ( nLen-- )
      {
        at( --nIndexCur ) = *--pSrcElm;
        *--pSrcElm = NULL;
      }
    }
  }
}


template<class TElm,class EParentAllocator>
inline void 
EAllocatorPtrT<TElm,EParentAllocator>::construct( SL nIndex )
{
  at( nIndex ) = NULL;
}


template<class TElm,class EParentAllocator>
inline void 
EAllocatorPtrT<TElm,EParentAllocator>::construct( SL nIndex,
                                                  SL nLen )
{
  EASSERT( nLen >= 0 );
  EASSERT( nIndex + nLen <= count() );
  while ( nLen-- )
    at( nIndex++ ) = NULL;
}


template<class TElm,class EParentAllocator>
inline void 
EAllocatorPtrT<TElm,EParentAllocator>::destruct( SL nIndex )
{
  TElm pPtr = at( nIndex );
  if ( pPtr  &&  ownership() )
    delete pPtr;
  at( nIndex ) = NULL;
}


template<class TElm,class EParentAllocator>
void 
EAllocatorPtrT<TElm,EParentAllocator>::destruct( SL nIndex, 
                                                 SL nLen )
{
  EASSERT( nLen >= 0 );
  EASSERT( nIndex + nLen <= count() );

  if ( ownership() )
  {
    while ( nLen-- )
    {
      TElm pPtr = at( nIndex );
      if ( pPtr )
        delete pPtr;
      at( nIndex++ ) = NULL;
    }
  }
  else
  {
    while ( nLen-- )
      at( nIndex++ ) = NULL;
  }
}


template<class TElm,class EParentAllocator>
inline void 
EAllocatorPtrT<TElm,EParentAllocator>::setOwnership( bool bOwnObject )
{
  _bOwnership = bOwnObject;
}


template<class TElm,class EParentAllocator>
inline bool 
EAllocatorPtrT<TElm,EParentAllocator>::ownership() const
{
  return _bOwnership;
}


template<class TElm,class EParentAllocator>
void 
EAllocatorPtrT<TElm,EParentAllocator>::copy( const EParentAllocator &allocator )
{
  setOwnership( false );
  EParentAllocator::copy( allocator );
}


#endif // !defined(AFX_EALLOCATORPTR_H__CB0A9032_BC16_11D2_91F2_004005446EE6__INCLUDED_)
