// EBaseCallback.h: interface for the EBaseCallback class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASECALLBACK_H__B78AA252_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_EBASECALLBACK_H__B78AA252_D476_11D2_BD55_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

/*! \class EBaseCallback 
 * \brief This class is the base class for all callbacks.
 *
 */
class EBaseCallback
{
public:
  /*! Constructs a non valid callback.
   */
  EBaseCallback(); 

  /*! Constructs a valid callback.
   * @param pObjReceiver Pointer on the object the message is sent to.
   *                     Must be \c NULL if the callback is a static method.
   * @param fnMember Member function of the object that must be called if
   *                 \nSizeFnMember is not null, otherwise pointer on the
   *                 static method that must be called.
   * @param nSizeFnMember Size of the pointer on the member function. If the
   *                      callback is a static method then \a nSizeFnMember
   *                      must be 0.
   */
  EBaseCallback( void *pObjReceiver,
                 const void *fnMember,
                 size_t nSizeFnMember );

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  bool isValid() const;


  union
  {
    // static function to call as a callback.
    const void *_fnCallback;

    // member fonction pointer.
    char _fnMemberCallback[ _ETOOLLIB_MAX_PTR_MEMBER_FUNCTION_SIZE ];
  };

  // object the message is send to.
  void *_pObjReceiver;
};


/*! \class ECallback
 * \brief This class provides a callback without parameters and return value.
 */
class ECallback : protected EBaseCallback
{
public:
  ECallback() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()() const;


  typedef void (*ECallerFn)(const EBaseCallback &cb);

  ECallback( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function >
class ECallbackCallerFn : public ECallback
{
public:
  ECallbackCallerFn( Function fnCallback );

  static void caller( const EBaseCallback &cb );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType>
inline ECallbackCallerFn< ReturnType (*)() >
eCbCreator( ECallback *, 
            ReturnType (*fn)() )
{
  return ECallbackCallerFn< ReturnType (*)() >( fn );
}

template< class Receiver, class Member >
class ECallbackCallerMember : public ECallback
{
public:
  ECallbackCallerMember( Receiver *pObjReceiver, 
                         const Member &fnMember );


  static void caller( const EBaseCallback &cb );
};




template< class Receiver, class ReturnType, class CallObjType >
inline ECallbackCallerMember< Receiver, ReturnType (CallObjType::*)() >
eCbCreator( ECallback *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)() )
{
  typedef ReturnType (CallObjType::*MemberCallback)();

  return ECallbackCallerMember< Receiver, MemberCallback >( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################

inline
EBaseCallback::EBaseCallback()
{
  _pObjReceiver =NULL;
}


inline bool 
EBaseCallback::isValid() const
{
  return _pObjReceiver != NULL;
}


inline 
ECallback::operator bool() const
{
  return isValid();
}


inline 
void 
ECallback::operator ()() const
{
  _fnCaller( *this );
}


inline
ECallback::ECallback( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function >
inline
ECallbackCallerFn<Function>::ECallbackCallerFn( Function fnCallback ) :
    ECallback( caller, NULL, fnCallback, 0 )
{
}


template< class Function >
inline void 
ECallbackCallerFn<Function>::caller( const EBaseCallback &cb )
{
  ((Function)cb._fnCallback)();
}


template< class Receiver, class Member >
inline 
ECallbackCallerMember<Receiver,Member>::ECallbackCallerMember( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member >
inline void
ECallbackCallerMember<Receiver,Member>::caller( const EBaseCallback &cb )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

  (objReceiver.*fnMember)();
}

#endif // !defined(AFX_EBASECALLBACK_H__B78AA252_D476_11D2_BD55_0040053AD648__INCLUDED_)
