// ###########################################################################
// File: EARRAY.HPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// version 0.3
// ###########################################################################


/* HST:
 * 04/04/98: - Use new allocator erealloc instead of realloc.
 *             Throw an exception on allocation failure !
 * 24/07/98: - Template now have a isObject parameter.
 *             Not some method (setBinaryMode) have been removed !!!
 *             => Compatibility break !
 *           - Attribute mBinaryMode removed !
 * 10/10/98: - BugFix: Bad assertion in move(). Tests if dst <=0 instead of
 *             dst >= 0.
 * 05/11/98: - BugFix: insert() did not insert item but replace them !
 */

/* Bug:
 * - Port to VC 5: -> #ifdef _MSC_VER, pb with operator[] solved.
 * - Filler: size=4 -> UW instead of UL !
 */

/* TODO:
 * - BugFix: If object are in the array, need to destroy in the destructor !!!
 *           and to call the constructor !
 * - replace ?
 * - Check for method deep compare/copy, binary compare/copy.
 * - change void append(...) to EArray<TElm,isObject>& append(...); et pareille
 *   pour d'autres methodes.
 * - Bug: override init() method is not called in constructor.
 */
#ifndef EARRAY_HPP_INCLUDED
#define EARRAY_HPP_INCLUDED
 
#include <string.h>
#include <stdlib.h>

//    Types:
// #####################Last change: 21/08/97 20h00mn#########################

/*! 
 * \brief Template class that provides arrays of either objects or simple types.
 * 
 * The template definition is as follow:
 * \code
 * template<class TElm,bool isObject =false>
 * class EArray;
 * \endcode
 * When an element is added or removed from the array then the array is reallocated.
 * If a more efficient memory management is needed, the class EDynArray should
 * be used instead.
 * 
 If \c isObject is true then element are copied and compared one by one using the
 *  \c operator \c = and \c operator \c ==.
 *
 *  If \c isObject is \c false then elements are copied using binary copy function (memcpy for
 *  large structure ), and compared using the \c operator \c ==.
 *
 *  @param TElm Type of items stored in the arry.
 *  @param isObject If \c true then items are copied using a copy operator, otherwise they
 *                  are copied using a binary copy.
 */
template<class TElm,bool isObject =false>
class EArray {
  public:
    /*! Alias for this type of array.
     */
    typedef EArray<TElm,isObject> EArrayType;

    /*! Default constructor
     */
    EArray();

    /*!
     * Constructs an array with a size large enough to store \a size element.
     * @param size initial size of the array
     */
    EArray( SL size );

    /*! Copy constructor.
      * Constructs a copy of the array \a a. 
      * each element of the array.
      * @param a array to copy.
      */
    EArray( const EArrayType &a );

    /*! Destructor
     */
    ~EArray();

    /*! Copy operator.
      * Copy the array \a a.
      * @param a array to copy.
      */
    EArray<TElm,isObject>& operator =( const EArrayType &a );

    /*! Return a reference to the element at position \a index in the array.
     * @param index position of the referenced element. 0 is the first element
     *              of the array.
     */
#ifdef _MSC_VER
    TElm& operator []( int index );
#else
    TElm& operator []( SL index );
#endif

    /*! Return a reference to the element at position \a index in the array.
     * @param index position of the referenced element. 0 is the first element
     *              of the array.
     */
#ifdef _MSC_VER
    const TElm& operator []( int index ) const;
#else
    const TElm& operator []( SL index ) const;
#endif

    /*! Return a reference to the element at position \a index in the array.
     * @param index position of the referenced element. 0 is the first element
     *              of the array.
     */
    const TElm& at( SL index ) const;

    /*! Return a reference to the element at position \a index in the array.
     * @param index position of the referenced element. 0 is the first element
     *              of the array.
     */
    TElm& at( SL index );

    /*! Return the number of elements in the array.
     * @return Number of elements in the array.
     */
    SL count() const;

    /*! Return the number of allocated elements for the array.
     * This is the number of elements that can be stored in the array
     * without any reallocation.
     *
     * The default return the count() of the array.
     * @return size of the number of allocated element in the array.
     */
    virtual SL maxSize() const;

    /*! Get a pointer on the first element of the array.
     * The returned pointer is the beginning of the array. As it
     * is stored like a standard C array you can read the array this way.
     * @return Return a pointer on the first element of the array.
     */
    operator const TElm *() const;

    /*! Get a pointer on the first element of the array.
     * The returned pointer is the beginning of the array. As it
     * is stored like a standard C array you can read or write the array this way.
     *
     * Beware that data() might returns \c NULL if the array contains 0 element.
     * 
     * This can be used to interface with function such as \c qsort.
     * @return Return a pointer on the first element of the array.
     */
    TElm *data();

    /*! Get a pointer on the first element of the array.
     * The returned pointer is the beginning of the array. As it
     * is stored like a standard C array you can read or write the array this way.
     *
     * Beware that data() might returns \c NULL if the array contains 0 element.
     * 
     * This can be used to interface with function such as \c qsort.
     * @return Return a pointer on the first element of the array.
     */
    const TElm *data() const;

    /*! Tests if the array is empty.
     * The array is empty if count() returns 0.
     * @return \c true if the array is empty, \c false otherwise.
     */
    bool isEmpty() const;

    /*! Tests if the array is null.
     * The array is null if count() returns 0 or data() returns \c NULL.
     * @return \c true if the array is NULL, \c false otherwise.
     */
    bool isNull() const;

    /*! Duplicate a slice of the array.
     * The returned array is a copy of the elements of this array beginning
     * at the position \a nIndex and of length \a nSize.
     *
     * The first element of the array is at position 0.
     *
     * If \a nSize is negative then the copy of elements stop at position
     * count()+nSize. This means that if \a nSize = -1 then the slice extends
     * until the end of this array. If \a nSize = -2 then the slice extends
     * until the element preceding the last one...
     *
     * @param nIndex Position of the first element to copy. 0 is the position
     *               of the first element
     * @param nSize Length of the slice.
     * @return An array that contains the slice.
     */
    EArrayType slice( SL nIndex, SL nSize =-1 ) const;

    /*! Resizes the array.
     * Shrinks or expands the array so it can hold \a size elements. The new
     * elements are unitialised. If a size of 0 is given then the array is
     * empty and null.
     * @param size New size of the array.
     * @param changeCount If the array is enlarged and changeCount is true,
     *        then the number of elements in the array is changed. If the new
     *        array size is then than the current the number of elementsn then
     *        the count is changed.
     */
    virtual void resize( SL size, bool changeCount =true );

    /*! Fill the array with a given element.
     * Fill the array with element \a filler. Resize the array before filling
     * if \a len if a positive number. The method used to copy the element
     * depends on the binary mode.
     * constructor.
     * @param filler element used to fill the array.
     * @param len resize the array if positive (>=0).
     * @see setBinaryMode
     */
    void fill( const TElm *filler, SL len =-1 );

    /*! Tests if two array are equal.
     * Test if the array is equal to array \a a. The comparaison method
     * depends on the binary mode.
     * @param a compared array.
     * @return true if the two array are equal.
     * @see setBinaryMode
     */
    bool operator ==(const EArray<TElm,isObject>& a) const;

    /*! Duplicate an array.
     * Copy the array \a a.
     * @param a array to copy.
     * @return reference on the array.
     */
    EArrayType& duplicate( const EArray<TElm,isObject>& a );

    /*! Duplicate an array.
     * Copy \a size element of the array \a a.
     * @param a array to copy.
     * @param size number of element to copy
     * @return reference on the array.
     */
    EArrayType& duplicate( const TElm *a, SL size );

    /*! Tests if two array are different.
     * Test if the array is different of array \a a. The comparaison method
     * depends on the binary mode.
     * @param a compared array.
     * @return true if the two array are different.
     * @see setBinaryMode
     */
    bool operator !=(const EArrayType &a) const;

    /*! Returns the number of time the item is present in the array.
     * Count the number of times \a elm occurs in the array. The comparaison
     * method depends on the binary mode.
     * @param elm item looked for.
     * @return number of times \a elm occurs in the array.
     * @see setBinaryMode
     */
    SL contains( const TElm *elm ) const;

    /*! Find the first occurence of elm given a starting position.
     * Return the position of the first occurence of \a elm, starting at
     * position \a index. Return -1 if \a elm could not be found.
     * The comparaison method depends on the binary mode.
     * @param elm element to find.
     * @param index position the search start.
     * @return position of the first element found. -1 it not found.
     * @see setBinaryMode
     */
    SL find( const TElm *elm, SL index =0 ) const;

    /*! Copy a slice of the array at another position.
     * Copy \a len elements of the array starting at position \a index
     * to position \a dst.
     * If \a len is negative then element are copied from position \a index to
     * position count()+len. So a value of -1 for length means that elements
     * are copied until the end of the array.
     * @param index
     * @param dst
     * @param len
     */
   void move( SL index, SL len, SL dst );

   /*! Removes somes elements from the array.
    * Removes elements from the array and resizes the array to the
    * right size.
    * If \a len is negative then element are removes from position \a index to
    * position count()+len. The default value of -1 means that elements are
    * removed from position \a index to the end of the array. That is
    * equivalent to truncate the array at position \a index.
    * @param index index of the first element to delete.
    * @param len number of elements of delete.
    */
   void remove( SL index, SL len =-1 );

   /*! Inserts a element in the array.
    * Insert element a in the array at position \a pos. If \pos is greater
    * than the length of the array then the array is extended. The extended
    * part of the array is left unitialised except for the inserted element.
    * @param index position of insertion of the element.
    * @param v element to insert.
    */
   void insert( SL index, const TElm& v );

   /*! Inserts another array in the array.
    * Insert the other array at position \a pos.
    * If \a pos is greater
    * than the length of the array then the array is extended. The extended
    * part of the array is left unitialised except for the inserted element.
    * @param index position of insertion of the element.
    * @param a array to insert.
    */
   void insert( SL index, const EArray<TElm,isObject>& a );

   /*! Append an element at the end of the array.
    * Enlarge the array by one element and insert the element at the end.
    * @param v element to insert.
    */
   void append( const TElm& v );

   /*! Append a array at the end of the array.
    * Append a array at the end of the array.
    * @param a array to append at the end.
    */
   void append( const EArray<TElm,isObject>& a );

   /*! Takes an element of the array.
    *
    * The elements that is taken is removed from the array. Therefore
    * the array is resized.
    *
    * @param nIdx Index of the element to take.
    * @return Element that is taken from the array.
    */
   TElm take( SL nIdx );

   /*! Takes an element of the array.
    *
    * The elements that is taken is removed from the array. Therefore
    * the array is resized.
    *
    * @param nIdx Index of the element to take.
    * @param v Element that is taken from the array, on return.
    */
   void take( SL nIdx,
              TElm &v );

   /*! Takes the first element of the array.
    *
    * @param v First element of the array.
    */
   TElm takeFirst();

   /*! Takes the first element of the array.
    *
    * @param v First element of the array, on return.
    */
   void takeFirst( TElm &v );

   /*! Takes the last element of the array.
    *
    * @return Last element of the array.
    */
   TElm takeLast();

   /*! Takes the last element of the array.
    *
    * @param v Last element of the array, on return.
    */
   void takeLast( TElm &v );

  protected:
    /*! Set the number of element in the array.
     * Set the number of element in the array.
     * @param count number of element.
     */
    void setCount( SL count );

    /*! Set the number of allocated element in the array.
     * Set the number of allocated element in the array.
     * Default set the number of element in the array to maxSize.
     * @param maxSize number of allocated element in the array.
     */
    virtual void setMaxSize( SL maxSize );

protected:
    //! initialise member.
    void init( SL size =0 );

private:

    //! Number of elements in the array.
    SL _nLength;

    //! data stored in the array
    TElm *_Datas;
};

// Implementation:
// #####################Last change: 21/08/97 22h40mn#########################

// initialise member
template<class TElm, bool isObject>
void EArray<TElm,isObject>::init( SL size )
{
  _nLength =0;
  _Datas =0;
  if ( size )
    resize( size );
}

// Default constructor
template<class TElm, bool isObject>
inline EArray<TElm,isObject>::EArray()
{
  init();
}

/* Constructs an array with a predefine size.
 * Constructs an array with a size to store \a size element.
 * @param size initial size of the array
 */
template<class TElm, bool isObject>
EArray<TElm,isObject>::EArray( SL size )
{
  init( size );
}

/* Copy constructor.
 * Constructs a copy of the array \a a. 
 * each element of the array.
 * @param a array to copy.
 */
template<class TElm, bool isObject>
EArray<TElm,isObject>::EArray( const EArrayType &a )
{
  init();
  duplicate( a );
}

// Destructor
template<class TElm, bool isObject>
EArray<TElm,isObject>::~EArray()
{
  if ( _Datas )
    efree( _Datas);
}

/* Copy operator.
  * Copy the array \a a. 
  * @param a array to copy.
  */
template<class TElm, bool isObject>
inline EArray<TElm,isObject>& EArray<TElm,isObject>::operator =( const EArray<TElm,isObject> &a )
{
  return duplicate( a );
}

/* Return a reference to the element at position \a index in the array.
 * @param index position of the referenced element. 0 is the first element
 *              of the array.
 */
template<class TElm, bool isObject>
#ifdef _MSC_VER
inline TElm& EArray<TElm,isObject>::operator []( int index )
#else
inline TElm& EArray<TElm,isObject>::operator []( SL index )
#endif
{
  EASSERT( index >= 0 );
  return _Datas[ index ];
}

/* Return a reference to the element at position \a index in the array.
 * @param index position of the referenced element. 0 is the first element
 *              of the array.
 */
template<class TElm, bool isObject>
#ifdef _MSC_VER
inline const TElm& EArray<TElm,isObject>::operator []( int index ) const
#else
inline const TElm& EArray<TElm,isObject>::operator []( SL index ) const
#endif
{
  EASSERT( index >= 0 );
  EASSERT( index < count() );
  return _Datas[ index ];
}

/* Return a reference to the element at position \a index in the array.
 * @param index position of the referenced element. 0 is the first element
 *              of the array.
 */
template<class TElm, bool isObject>
inline TElm& EArray<TElm,isObject>::at( SL index )
{
  EASSERT( index >= 0 );
  EASSERT( index < count() );
  return _Datas[ index ];
}

/* Return a reference to the element at position \a index in the array.
 * @param index position of the referenced element. 0 is the first element
 *              of the array.
 */
template<class TElm, bool isObject>
inline const TElm& EArray<TElm,isObject>::at( SL index ) const
{
  EASSERT( index >= 0 );
  EASSERT( index < count() );
  return _Datas[ index ];
}

/* Return the size of the array.
  * Return the size of the array (number of element in the array).
  * @return size of the array.
  */
template<class TElm, bool isObject>
inline SL EArray<TElm,isObject>::count() const
{
  return _nLength;
}

/* Return the number of allocated elements for the array.
  * Return the number of allocated element for the array. This size can be
  * greater than the number of element in the array if the array use some
  * special allocation strategy when resizing (EDynaArray for example).
  * The default return the count() of the array.
  * @return size of the number of allocated element in the array.
  */
template<class TElm, bool isObject>
inline SL EArray<TElm,isObject>::maxSize() const
{
  return _nLength;
}

/* Return a pointer on the array.
 * Cast operator. Return a pointer on the array.
 * @return Return a pointeur on the array.
 */
template<class TElm, bool isObject>
inline EArray<TElm,isObject>::operator const TElm *() const
{
  return _Datas;
}

/* Return a pointer on the data array.
 * Return a pointer on the data array.
 * @return Return a pointeur on the array.
 */
template<class TElm, bool isObject>
inline TElm *EArray<TElm,isObject>::data()
{
  return _Datas;
}

/* Return a pointer on the data array.
 * Return a pointer on the data array.
 * @return Return a pointeur on the array.
 */
template<class TElm, bool isObject>
inline const TElm *EArray<TElm,isObject>::data() const
{
  return _Datas;
}

/* Test if the array is empty.
 * Test if the array is empty: \l count() =0.
 * @return true if the Array is empty, false otherwise.
 */
template<class TElm, bool isObject>
inline bool EArray<TElm,isObject>::isEmpty() const
{
  return count() == 0;
}

/* Test if the array is null.
 * Test if the array is null: \l count() =0 and data() =0.
 * @return true if the Array is null, false otherwise.
 */
template<class TElm, bool isObject>
inline bool EArray<TElm,isObject>::isNull() const
{
  return count() == 0  &&  _Datas == 0;
}

/* Return a slice of an array.
 * Return a slice of the array: A array with \a len element from position
 * \a index. Position 0 is the first element. If \a len is negative then
 * all element are copied from position \a index to count()+len. The
 * default value for \a len is -1, so all element are copied until the
 * end of the array. The m
 * The method use to copy the element depends on the binary mode.
 * @param index position of the first element to copy
 * @param len length of the slice
 * @return a array that hold the slice..
 * @see setBinaryMode
 */
template<class TElm, bool isObject>
EArray<TElm,isObject> EArray<TElm,isObject>::slice( SL index, SL len ) const
{
  EASSERT( index >= 0 );
  SL endPos = len >= 0 ? index + len : max( index, count() + len +1 );
  SL sz = max( endPos -index, 0 );
  EArray<TElm,isObject> a( sz );

  if ( !isObject )
    memcpy( a.data(), data(), sizeof(TElm) *sz );
  else
  {
    SL cpt;
    for ( cpt =0; cpt <sz; ++cpt )
      a[cpt] = at( index +cpt );
  }

  return a;
}

/* Resizes the array.
  * Shrinks or expands the array so it can hold \a size elements. The new
  * elements are unitialised. If a size of 0 is given then the array is
  * empty and null.
  * @param size new size of the array.
  */
template<class TElm, bool isObject>
void EArray<TElm,isObject>::resize( SL size, bool changeCount )
{
  EASSERT( size >= 0 );
  if ( size == 0 )
  {
    efree( _Datas );
    _Datas =0;
    setMaxSize( 0 );
    setCount( 0 );
  }
  else
  {
    _Datas = (TElm *)erealloc( _Datas, size *sizeof(TElm) );
    setMaxSize( size );
    if ( changeCount  ||  size < count() )
      setCount( size );
  }
}

/* Fill the array.
  * Fill the array with element \a filler. Resize the array before filling
  * if \size if a positive number. The method used to copy the element
  * depends on the binary mode.
  * constructor.
  * @param filler element used to fill the array.
  * @param size resize the array if positive (>=0).
  * @see setBinaryMode
  */

template<class TElm, bool isObject>
void EArray<TElm,isObject>::fill( const TElm *filler, SL len )
{
  EASSERT( len >= -1 );

  if ( len >= 0 )
    resize( len );

  if ( !isObject )
  {
    const char *f = (const char *)filler;

    if ( sizeof( TElm ) == 1 )
      memset( _Datas, *f, count() );
    else
    {
      SL len = count();

      if ( sizeof( TElm ) == 2 )
      {
        UW *p = (UW *)_Datas;
        while ( len-- )
          *p++ = *(UW*)f;
      }
      else
      {
        if ( sizeof( TElm ) == 4 )
        {
          UL *p = (UL *)_Datas;
          while ( len-- )
            *p++ = *(UL*)f;
        }
        else
        {
          UB *p = (UB *)_Datas;
          while ( len-- )
          {
            memcpy( p, f, sizeof( TElm ) );
            p += sizeof( TElm );
          }
        }
      }
    }
  }
  else
  {
    SL len = count();
    while ( len-- )
      at( len ) = *filler;
  }
}

/* Tests if two array are equal.
  * Test if the array is equal to array \a a. The comparaison method
  * depends on the binary mode.
  * @param a compared array.
  * @return true if the two array are equal.
  * @see setBinaryMode
  */
template<class TElm, bool isObject>
bool EArray<TElm,isObject>::operator ==(const EArray<TElm,isObject>& a) const
{
  if ( count() != a.count() )
    return false;

  if ( !isObject )
    return memcmp( _Datas, a.data(), sizeof( TElm ) * count() ) == 0;
  else
  {
    SL len = count();
    while ( len-- )
    {
      if ( at( len ) != a.at( len ) )
        return false;
    }
    return true;
  }
}

/* Tests if two array are different.
  * Test if the array is different of array \a a. The comparaison method
  * depends on the binary mode.
  * @param a compared array.
  * @return true if the two array are different.
  * @see setBinaryMode
  */
template<class TElm, bool isObject>
inline bool EArray<TElm,isObject>::operator !=(const EArray<TElm,isObject>& a) const
{
  return !( *this == a );
}

/* Returns the number of time the item is present in the array.
  * Count the number of times \a elm occurs in the array. 
  * @param elm item looked for.
  * @return number of times \a elm occurs in the array.
  */
template<class TElm, bool isObject>
SL EArray<TElm,isObject>::contains( const TElm *elm ) const
{
  SL num =0;
  SL len = count();

  if ( !isObject )
  {
    const TElm *p = _Datas;
    while ( len-- )
    {
      if ( memcmp( p++, elm, sizeof( TElm ) ) == 0 )
        num++;
    }
  }
  else
  {
    const TElm *p = _Datas;
    while ( len-- )
    {
      if ( *p++ == *elm )
        num++;
    }
  }

  return num;
}

/* Find the first occurence of elm given a starting position.
  * Return the position of the first occurence of \a elm, starting at
  * position \a index. Return -1 if \a elm could not be found.
  * @param elm element to find.
  * @param index position the search start.
  * @return position of the first element found. -1 it not found.
  */
template<class TElm, bool isObject>
SL EArray<TElm,isObject>::find( const TElm *elm, SL index ) const
{
  EASSERT( index >= 0 );

  if ( index >= count() )
    return -1;

  SL pos = index;
  const TElm *p = &at(pos);

  if ( !isObject )
  {
    for (; pos < count(); ++pos)
    {
      if ( memcmp( p++, elm, sizeof( TElm ) ) == 0 )
        return pos;
    }
  }
  else
  {
    for (; pos < count(); ++pos)
    {
      if ( *p++ == *elm)
        return pos;
    }
  }

  return -1;
}

/* Duplicate an array.
  * Copy the array \a a.
  * @param a array to copy.
  * @return reference on the array.
  */
template<class TElm, bool isObject>
EArray<TElm,isObject>& EArray<TElm,isObject>::duplicate( const EArray<TElm,isObject>& a )
{
  resize( a.count() );
  if ( !isObject )
    memcpy( data(), a.data(), count() * sizeof( TElm ) );
  else
  {
    SL len = count();
    TElm *dst = data();
    const TElm *src = a.data();

    while ( len-- )
      *dst++ = *src++;
  }
  return *this;
}

/* Duplicate an array.
  * Copy \a size element of the array \a a.
  * @param a array to copy.
  * @param size number of element to copy
  * @return reference on the array.
  */
template<class TElm, bool isObject>
EArray<TElm,isObject>& EArray<TElm,isObject>::duplicate( const TElm *a, SL size )
{
  resize( size );
  if ( !isObject )
    memcpy( data(), a, count() * sizeof( TElm ) );
  else
  {
    SL len = count();
    TElm *dst = data();
    const TElm *src = a;

    while ( len-- )
      *dst++ = *src++;
  }
  return *this;
}

/* Copy a slice of the array at another position.
  * Copy \a len elements of the array starting at position \a index
  * to position \a dst.
  * If \a len is negative then element are copied from position \a index to
  * position count()+len. So a value of -1 for length means that elements
  * are copied until the end of the array.
  * @param index
  * @param dst
  * @param len
  */
template<class TElm, bool isObject>
void EArray<TElm,isObject>::move( SL index, SL len, SL dst )
{
  EASSERT( index >= 0 );
  EASSERT( dst >= 0 );
  SL endPos = len >= 0 ? index + len : EMAX( index, count() + len +1 );
  SL sz = EMIN( EMAX( endPos -index, 0 ), count() -dst);

  if ( !sz )
    return;

  if ( !isObject )
    memmove( &at( dst ), &at( index ), sizeof(TElm) *sz );
  else
  {
    SL cpt;
    if ( index < dst )
      for ( cpt =0; cpt <sz; ++cpt )
        at( dst +cpt ) = at( index +cpt );
    else
      for ( cpt =sz-1; cpt >= 0; --cpt )
        at( dst +cpt ) = at( index +cpt );
  }
}

/* Removes somes elements from the array.
 * Removes elements from the array and resizes the array to the
 * right size.
 * If \a len is negative then element are removes from position \a index to
 * position count()+len. The default value of -1 means that elements are
 * removed from position \a index to the end of the array. That is
 * equivalent to truncate the array at position \a index.
 * @param index index of the first element to delete.
 * @param len number of elements of delete.
 */
template<class TElm, bool isObject>
void EArray<TElm,isObject>::remove( SL index, SL len )
{
  EASSERT( index >= 0 );
  SL endPos = len >= 0 ? index + len : EMAX( index, count() + len +1 );
  SL sz = EMAX( endPos -index, 0 );

  move( endPos, -1, index );
  resize( count() -sz );
}

/* Inserts a element in the array.
 * Insert element a in the array at position \a pos. If \pos is greater
 * than the length of the array then the array is extended. The extended
 * part of the array is left unitialised except for the inserted element.
 * @param index position of insertion of the element.
 * @param v element to insert.
 */
template<class TElm, bool isObject>
void EArray<TElm,isObject>::insert( SL index, const TElm& v )
{
  EASSERT( index >= 0 );
  if ( index >= count() )
    resize( index +1 );
  else
  {
    resize( count() +1 );
    move( index, -1, index +1 );
  }
  at( index ) = v;
}

/* Inserts another array in the array.
 * Insert the other array at position \a pos.
 * If \a pos is greater
 * than the length of the array then the array is extended. The extended
 * part of the array is left unitialised except for the inserted element.
 * @param index position of insertion of the element.
 * @param a array to insert.
 */
template<class TElm, bool isObject>
void EArray<TElm,isObject>::insert( SL index, const EArray<TElm,isObject>& a )
{
  EASSERT( index >= 0 );
  if ( index +a.count() >= count() )
    resize( index +a.count() );
  else
  {
    resize( count + a.count() );
    move( index, -1, index+a.count() );
  }

  if ( !isObject )
    memcpy( &at( index ), a.data(), sizeof( TElm ) *a.count() );
  else
  {
    SL cpt;
    for ( cpt = a.count()-1; cpt >= 0; --cpt )
      at( index +cpt ) = a.at( cpt );
  }
}

/* Append an element at the end of the array.
 * Enlarge the array by one element and insert the element at the end.
 * @param v element to insert.
 */
template<class TElm, bool isObject>
inline void EArray<TElm,isObject>::append( const TElm& v )
{
  insert( count(), v );
}

/* Append a array at the end of the array.
 * Append a array at the end of the array.
 * @param a array to append at the end.
 */
template<class TElm, bool isObject>
inline void EArray<TElm,isObject>::append( const EArray<TElm,isObject>& a )
{
  insert( count(), a );
}

/* Set the number of element in the array.
 * Set the number of element in the array.
 * @param count number of element.
 */
template<class TElm, bool isObject>
inline void EArray<TElm,isObject>::setCount( SL count )
{
  _nLength = count;
}

/* Set the number of allocated element in the array.
 * Set the number of allocated element in the array.
 * Default set the number of element in the array to maxSize.
 * @param maxSize number of allocated element in the array.
 */
template<class TElm, bool isObject>
inline void EArray<TElm,isObject>::setMaxSize( SL maxSize )
{
  _nLength = maxSize;
}


template<class TElm, bool isObject>
TElm
EArray<TElm,isObject>::take( SL nIdx )
{
  TElm v = at( nIdx );
  remove( nIdx, 1 );
  return v;
}

template<class TElm, bool isObject>
void 
EArray<TElm,isObject>::take( SL nIdx,
          TElm &v )
{
  v = at( nIdx );
  remove( nIDx, 1 );
}

template<class TElm, bool isObject>
inline TElm
EArray<TElm,isObject>::takeFirst()
{
  return take( 0 );
}

template<class TElm, bool isObject>
inline void 
EArray<TElm,isObject>::takeFirst( TElm &v )
{
  take( 0, v );
}

template<class TElm, bool isObject>
inline TElm
EArray<TElm,isObject>::takeLast()
{
  return take( count()-1 );
}

template<class TElm, bool isObject>
inline void 
EArray<TElm,isObject>::takeLast( TElm &v )
{
  take( count()-1, v );
}

#endif
