// ###########################################################################
// File: EBINTREE.HPP
// (c)Copyright 08/1998, Baptiste Lepilleur (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef EBINTREE_HPP_INCLUDED

/* Test EBinTree<Node>:
 * - [OK] leftNode
 * - [OK] rightNode
 * - [OK] add
 * - [OK] root
 * - [OK] findMin
 * - [OK] findMax
 * - [OK] searchParent
 * - [OK] findLSTParent
 * - [OK] findRSTParent
 * - [OK] count
 * - [OK] setTree
 * - [OK] remove
 * - [  ] removeAll
 * - [OK] take
 */


// Includes:
// #####################Last change: 29/03/98 21h24mn#########################


// Constantes:
// #####################Last change: 29/03/98 21h24mn#########################


// Macros:
// #####################Last change: 29/03/98 21h24mn#########################

/** You should use this macro when you derive the class EBinTreeNode.
 * This macro will implement the basic mover leftST(), rightST() and
 * parentNode() with the right return type.
 * @param Type type of your node class.
 */
#define EBINTREENODE_MOVER( Type ) \
  Type *leftST() const { return (Type*)EBinTreeNode::leftST(); } \
  Type *rightST() const { return (Type*)EBinTreeNode::rightST(); } \
  Type *parentNode() const { return (Type*)EBinTreeNode::parentNode(); }

// Types:
// #####################Last change: 29/03/98 21h24mn#########################

// EBinTreeNode
// ***************************************************************************

/** A node for a binary tree.
 * The class that implement the node for the binary tree must be derivated
 * from this one.
 * The derivated class must implement the method:
 *  bool isLE( ClassName *pNode ) const.
 * This method return true if the current node is lower or equal to node.
 */
class EBinTreeNode
{
public:
  /// Constructs a binary tree item.
  EBinTreeNode();

  virtual ~EBinTreeNode() {}

  /** Get left sub tree.
   * @return left sub tree.
   */
  EBinTreeNode *leftST() const;

  /** Get right sub tree.
   * @return right sub tree.
   */
  EBinTreeNode *rightST() const;

  /** Get parent node.
   * @return parent node.
   */
  EBinTreeNode *parentNode() const;

  /** Set left and right sub tree.
   * @param pLST left sub tree.
   * @param pRST right sub tree.
   */
  void setSubtree( EBinTreeNode *pLST,
                   EBinTreeNode *pRST );
  /** Set left sub tree.
   * @param pLST left sub tree.
   */
  void setLST( EBinTreeNode *pLST );

  /** Set right sub tree.
   * @param pRST right sub tree.
   */
  void setRST( EBinTreeNode *pRST );

  /** Set parent node.
   * @param pParentNode parent node.
   */
  void setParentNode( EBinTreeNode *pParentNode );

  /** Make the node a leaf.
   */
  void setLeaf();

  /** Test if the node is a leaf.
   * @return true if the node is a leaf.
   */
  bool isLeaf() const;

  /** Remove all node of a subtree.
   * @param pNode subtree root.
   */
  void removeSubtree();

  /** Test if a node is less or equal than the other.
   * @param node we compare to.
   * @return true if this node is <= node.
   */
//  bool isLE( EBinTreeNode &node ) const;

private:

  /** Recursing removing all node of a subtree.
   * @param pNode subtree root.
   */
  void removeSubtreeRecurse();

  /// Left SubTree
  EBinTreeNode *_pLST;

  /// Right SubTree
  EBinTreeNode *_pRST;

  /// Parent Node
  EBinTreeNode *_pParentNode;
};

// EBinTree<Node>
// ***************************************************************************

/*!
 * \brief Template class that implements a simple binary tree.
 * 
 * The template definition is as follow:
 * \code
 * template<class Node>
 * class EBinTree;
 * \endcode
 * @param Node class for the nodes of the tree.
 */
template<class Node>
class EBinTree
{
public:
  /// Constructs an empty binary tree.
  EBinTree();

  /** Add a node in the tree.
   * @param pNode node to add in the tree.
   */
  void add( Node *pNode );

  /** Remove a node from the tree.
   * @param pNode node to remove.
   */
  void remove( Node *pNode );

  /** Take a node from the tree.
    * @param pNode node to remove.
    */
  Node *take( Node *pNode );

  /** Get the node with the leftward node (the smaller one).
   * @param pNode node the search begin at.
   * @return leftward node.
   */
  Node *findMin( Node *pNode =ENIL ) const;

  /** Get the node with the rightward node (the greater one).
   * @param pNode node the search begin at.
   * @return rightward node.
   */
  Node *findMax( Node *pNode =ENIL ) const;

  /** Search parent node.
   * @param pNode node we search the parent node.
   * @param bSmaller on return, if true then the node is on the left of the
   *                 parent node.
   * @return parent node.
   */
  Node *searchParent( Node *pNode,
                      bool &bSmaller  ) const;

  /** Get the number of node in the tree.
   * @return number of node in the tree.
   */
  SL count() const;

  /** Get the node that is on the right of this one.
   * @param pNode node we want the right node.
   * @return the node on the right. ENIL if none.
   */
  Node *rightNode( Node *pNode ) const;

  /** Get the root node.
   * @return root node.
   */
  Node *root() const;

  /** Set the root node (The tree must be empty!).
   * @param pRoot root node.
   * @param nNbNode number of node in the tree.
   */
  void setTree( Node *pRoot, SL nNbNode );

  /** Find the first parent for which the node is on its left subtree.
   * @param pNode node we search the special parent.
   * @return special parent if found, ENIL otherwise.
   */
  Node *findLSTParent( Node *pNode ) const;

  /** Get the node that is on the left of this one.
    * @param pNode node we want the left node.
    * @return the node on the left. ENIL if none.
    */
  Node *leftNode( Node *pNode ) const;

  /** Find the first parent for which the node is on its right subtree.
   * @param pNode node we search the special parent.
   * @return special parent if found, ENIL otherwise.
   */
  Node *findRSTParent( Node *pNode ) const;

  /** Remove all node from the tree.
   */
  void removeAll();
private:
  /**@shapeType AggregationLink*/
  /*#  EBinTreeNode pRoot*/
  /// root of the tree.
  Node *_pRoot;

  /// Number of node in the tree.
  SL _nNbNode;
};

// inlines:
// #####################Last change: 29/03/98 21h24mn#########################

// EBinTree
// ***************************************************************************
inline EBinTreeNode::EBinTreeNode()
{
  _pLST = ENIL;
  _pRST = ENIL;
  _pParentNode = ENIL;
}


/** Get left sub tree.
  * @return left sub tree.
  */
inline EBinTreeNode *EBinTreeNode::leftST() const
{
  return _pLST;
}

/** Get right sub tree.
  * @return right sub tree.
  */
inline EBinTreeNode *EBinTreeNode::rightST() const
{
  return _pRST;
}

/** Set left and right sub tree.
  * @param pLST left sub tree.
  * @param pRST right sub tree.
  */
inline void EBinTreeNode::setSubtree( EBinTreeNode *pLST,
                                      EBinTreeNode *pRST )
{
  _pLST = pLST;
  _pRST = pRST;
}
/** Set left sub tree.
  * @param pLST left sub tree.
  */
inline void EBinTreeNode::setLST( EBinTreeNode *pLST )
{
  _pLST = pLST;
}

/** Set right sub tree.
  * @param pRST right sub tree.
  */
inline void EBinTreeNode::setRST( EBinTreeNode *pRST )
{
  _pRST = pRST;
}

/** Get parent node.
  * @return parent node.
  */
inline EBinTreeNode *EBinTreeNode::parentNode() const
{
  return _pParentNode;
}

/** Set parent node.
  * @param pParentNode parent node.
  */
inline void EBinTreeNode::setParentNode( EBinTreeNode *pParentNode )
{
  _pParentNode = pParentNode;
}

/** Make the node a leaf.
  */
inline void EBinTreeNode::setLeaf()
{
  _pLST = ENIL;
  _pRST = ENIL;
}

/** Test if the node is a leaf.
  * @return true if the node is a leaf.
  */
inline bool EBinTreeNode::isLeaf() const
{
  return ( leftST() == ENIL  &&  rightST() == ENIL );
}

// EBinTree
// ***************************************************************************

/** Remove a node from the tree.
  * @param pNode node to remove.
  */
template<class Node>
inline void EBinTree<Node>::remove( Node *pNode )
{
  delete take( pNode );
}

/** Get the root node.
  * @return root node.
  */
template<class Node>
inline Node *EBinTree<Node>::root() const
{
  return _pRoot;
}

/** Get the number of node in the tree.
  * @return number of node in the tree.
  */
template<class Node>
inline SL EBinTree<Node>::count() const
{
  return _nNbNode;
}

// Template implentation:
// #####################Last change: 29/03/98 21h24mn#########################

/// Constructs an empty binary tree.
template<class Node>
EBinTree<Node>::EBinTree()
{
  _pRoot = ENIL;
  _nNbNode = 0;
}

/** Search parent node.
  * @param pNode node we search the parent node.
  * @param bSmaller on return, if true then the node is on the left of the
  *                 parent node.
  * @return parent node.
  */
template<class Node>
Node *EBinTree<Node>::searchParent( Node *pNode,
                                    bool &bSmaller ) const
{
  EASSERT( pNode );

  if ( !_pRoot )
    return ENIL;

  Node *pCurNode = _pRoot;
  while ( pCurNode )
  {
    Node *pSubNode;
    bSmaller = pNode->isLE( *pCurNode );
    if ( bSmaller )   // go down the left subtree
    {
      pSubNode = (Node *)pCurNode->leftST();
    }
    else
      pSubNode = (Node *)pCurNode->rightST();

    if ( !pSubNode )
      break;
    pCurNode = pSubNode;
  }
  return pCurNode;
}

/** Add a node in the tree.
  * @param pNode node to add in the tree.
  */
template<class Node>
void EBinTree<Node>::add( Node *pNode )
{
  bool bSmaller;
  Node *pParent = searchParent( pNode, bSmaller );

  pNode->setLeaf();
  pNode->setParentNode( pParent );

  if ( !pParent )     // tree is empty, node is root.
    _pRoot = pNode;
  else
  {
    if ( bSmaller )     // on the left ?
      pParent->setLST( pNode );
    else
      pParent->setRST( pNode );
  }

  _nNbNode++;
}

/** Take a node from the tree.
  * @param pNode node to remove.
  */
template<class Node>
Node *EBinTree<Node>::take( Node *pNode )
{
  EASSERT( pNode );

  Node *pParent = (Node *)pNode->parentNode();

  if ( pNode->isLeaf() )      // leaf node
  {
    if ( pParent )
    {
      if ( pParent->leftST() == pNode )   // on the left of the parent ?
        pParent->setLST( ENIL );
      else
        pParent->setRST( ENIL );
    }
    else
      _pRoot = ENIL;
  }
  else if ( pNode->leftST()  && !pNode->rightST() )
  {                                       // only have a child on the left
    Node *pSubNode = (Node *)pNode->leftST();
    pSubNode->setParentNode( pParent );
    if ( pParent )
    {
      if ( pParent->leftST() == pNode )   // on the left of the parent ?
        pParent->setLST( pSubNode );
      else
        pParent->setRST( pSubNode );
    }
    else
      _pRoot = pSubNode;
  }
  else if ( pNode->rightST()  &&  !pNode->leftST() )
  {                                       // only have a child on the right
    Node *pSubNode = (Node *)pNode->rightST();
    pSubNode->setParentNode( pParent );
    if ( pParent )
    {
      if ( pParent->leftST() == pNode )   // on the left of the parent ?
        pParent->setLST( pSubNode );
      else
        pParent->setRST( pSubNode );
    }
    else
      _pRoot = pSubNode;
  }
  else                                 // have two child, on the right and left.
  {
    Node *pRightNode = (Node *)pNode->rightST();
    Node *pMinNode = findMin( pRightNode );
    take( pMinNode );
    _nNbNode++;

    Node *pLeftNode = (Node *)pNode->leftST();
    if ( pLeftNode )
      pLeftNode->setParentNode( pMinNode );

    pRightNode = (Node *)pNode->rightST();  // taking pMinNode may change it !
    if ( pRightNode )
      pRightNode->setParentNode( pMinNode );

    pMinNode->setSubtree( pLeftNode, pRightNode );
    pMinNode->setParentNode( pParent );
    if ( pParent )
    {
      if ( pParent->leftST() == pNode )
        pParent->setLST( pMinNode );
      else
        pParent->setRST( pMinNode );
    }
    else
      _pRoot = pMinNode;
  }

  _nNbNode--;
  return pNode;
}

/** Get the node with the leftward node (the smaller one).
  * @param pNode node the search begin at.
  * @return leftward node.
  */
template<class Node>
Node *EBinTree<Node>::findMin( Node *pNode ) const
{
  if ( !_pRoot )
    return ENIL;

  if ( !pNode )
    pNode = _pRoot;

  Node *pSubNode;

  while ( pSubNode = (Node *)pNode->leftST() )
    pNode = pSubNode;

  return pNode;
}

/** Get the node with the rightward node (the greater one).
  * @param pNode node the search begin at.
  * @return rightward node.
  */
template<class Node>
Node *EBinTree<Node>::findMax( Node *pNode) const
{
  if ( !_pRoot )
    return ENIL;

  if ( !pNode )
    pNode = _pRoot;

  Node *pSubNode;

  while ( pSubNode = (Node *)pNode->rightST() )
    pNode = pSubNode;

  return pNode;
}

/** Find the first parent for which the node is on its left subtree.
 * @param pNode node we search the special parent.
 * @return special parent if found, ENIL otherwise.
 */
template<class Node>
Node *EBinTree<Node>::findLSTParent( Node *pNode ) const
{
  Node *pParent;

  while( (pParent = (Node *)pNode->parentNode())  &&
         (pParent->leftST() !=  pNode ) )
  {
    pNode = pParent;
  }
  return pParent;
}

/** Find the first parent for which the node is on its right subtree.
 * @param pNode node we search the special parent.
 * @return special parent if found, ENIL otherwise.
 */
template<class Node>
Node *EBinTree<Node>::findRSTParent( Node *pNode ) const
{
  Node *pParent;

  while( (pParent = (Node *)pNode->parentNode())  &&
         (pParent->rightST() !=  pNode ) )
  {
    pNode = pParent;
  }
  return pParent;
}

/** Get the node that is on the right of this one.
  * @param pNode node we want the right node.
  * @return the node on the right. ENIL if none.
  */
template<class Node>
Node *EBinTree<Node>::rightNode( Node *pNode ) const
{
  Node *pParent = (Node *)pNode->parentNode();

  if ( pNode->isLeaf() )
  {
    if ( pParent )
    {
      if ( pParent->leftST() == pNode )  // leaf && parent->LST
        return pParent;
      else                              // leaf && parent->RST
        return findLSTParent( pNode );
    }
    else    // root & leaf
      return ENIL;
  }
  else if ( pNode->leftST()  &&  !pNode->rightST() )  // single && LST
  {
    return findLSTParent( pNode );
  }
  else        // double  ||  single, RST
  {
    return findMin( (Node *)pNode->rightST() );
  }
}

/** Get the node that is on the left of this one.
  * @param pNode node we want the left node.
  * @return the node on the left. ENIL if none.
  */
template<class Node>
Node *EBinTree<Node>::leftNode( Node *pNode ) const
{
  Node *pParent = (Node *)pNode->parentNode();
  if ( pNode->isLeaf() )
  {
    if ( pParent )
    {
      if ( pParent->leftST() == pNode )   // leaf & parent->LST
        return findRSTParent( pNode );
      else                                // leaf & parent->RST
        return pParent;
    }
    else    // root & leaf
      return ENIL;
  }
  else if ( pNode->leftST()  &&  !pNode->rightST() )  // single, LST
  {
    return findMax( (Node *)pNode->leftST() );
  }
  else if ( pNode->rightST()  &&  !pNode->leftST() )  // single, RST
  {
    if ( pParent )
      if ( pParent->rightST() == pNode )  // single, RST && parent->RST
        return pParent;
    return ENIL;
  }
  else      // double
  {
    return findMax( (Node *)pNode->leftST() );
  }
}

/** Set the root node (The tree must be empty!).
  * @param pRoot root node.
  * @param nNbNode number of node in the tree.
  */
template<class Node>
void EBinTree<Node>::setTree( Node *pRoot, SL nNbNode )
{
  EASSERT( _pRoot == ENIL );
  _pRoot = pRoot;
  _nNbNode = nNbNode;
}

/** Remove all node from the tree.
  */
template<class Node>
void EBinTree<Node>::removeAll()
{
  if ( root() )
    root()->removeSubtree();
  _pRoot = ENIL;
  _nNbNode = 0;
}

#define EBINTREE_HPP_INCLUDED
#endif


