// ###########################################################################
// File: EBITSTRM.CPP
// (c)Copyright 04/1998, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################

#include "usuel.hpp"
#include "ebitstrm.hpp"

/* HST:
 * 05/04/98: -bug fix: getByte(), throughtly bugged !
 *           -bug fix: putByte().
 * 06/04/98: -bug fix: readBlock() & writeBlock()
 *           -bug fix: operator <<( const EBitArray& aBits )
 */

/* Test:
 * [OK] EBitStream( EIODevice *pDevice )
 * [  ] EBitStream()
 * [  ] ~EBitStream()
 * [OK] bool getBit()
 * [OK] void putBit( bool bBit =1)
 * [OK] void readBlock( void *pData, SL nLen )
 * [OK] void writeBlock( const void *pData, SL nLen )
 * [OK] UB getByte()
 * [OK] void putByte( UB v)
 * [  ] EBitStream& operator >>(SB& v)
 * [  ] EBitStream& operator >>(SW& v)
 * [  ] EBitStream& operator >>(SL& v)
 * [  ] EBitStream& operator >>(UB& v)
 * [  ] EBitStream& operator >>(UW& v)
 * [  ] EBitStream& operator >>(UL& v)
 * [  ] EBitStream& operator >>(float& v)
 * [  ] EBitStream& operator >>(double& v)
 * [  ] EBitStream& operator >>(char *& v)
 * [  ] EBitStream& operator <<(SB v)
 * [  ] EBitStream& operator <<(SW v)
 * [  ] EBitStream& operator <<(SL v)
 * [  ] EBitStream& operator <<(UB v)
 * [  ] EBitStream& operator <<(UW v)
 * [  ] EBitStream& operator <<(UL v)
 * [  ] EBitStream& operator <<(float v)
 * [  ] EBitStream& operator <<(double v)
 * [  ] EBitStream& operator <<(const char * v)
 * [OK] EBitStream& operator <<( const EBitArray& aBits )
 * [OK] void init()
 * [OK] void flush()
 * [OK] void readFlush()
 * [OK] void writeFlush()
 */

/// Initialize members.
void EBitStream::init()
{
  _nShift = 0;
  _nByte = 0;
  _bReadMode = true;
}

/** Set device used by the bit stream.
  * Set the device used by the bit stream.
  * @param pDevice device used by the bit stream.
  */
void EBitStream::setDevice( EIODevice *pDevice )
{
  assert( pDevice );

  EStream::setDevice( pDevice );

  _bReadMode = ( pDevice->isReadWrite()  ||  pDevice->isReadable() );
  if ( isReadMode() )
  {
    _nShift = 8;
    readFlush();
  }
  else
  {
    _nShift = 0;
    _nByte = 0;
  }
}

/** Constructs a data stream with a device.
  * Constructs a data stream for device \a device.
  * @param pDevice device used by the data stream.
  */
EBitStream::EBitStream( EIODevice *pDevice ) :
  EStream( pDevice )
{
  init();
  setDevice( pDevice );
}

/// Default constructor.
EBitStream::EBitStream()
{
  init();
}

/// Destructor.
EBitStream::~EBitStream()
{
  flush();
}

/// Write the last bit if needed.
void EBitStream::flush()
{
  assert( device() );

  if ( !isReadMode()  &&  _nShift > 0 )
    device()->putByte( _nByte );
}

/** Read a bit from the stream.
  * @return true if the bit value is 1, false if it is 0.
  */
bool EBitStream::getBit()
{
  readFlush();
  return ((_nByte >> _nShift++) & 1) != 0;
}

/** Write a bit to the stream.
  * @param bBit if it is true then a bit of value 1 is written, 0 otherwise.
  */
void EBitStream::putBit( bool bBit )
{
  writeFlush();
  if ( bBit )
    _nByte |= 1 << _nShift;
  _nShift++;
}

/** Read a block.
  * @param pData data to read.
  * @param nLen length of data to read in byte.
  */
void EBitStream::readBlock( void *pData, SL nLen )
{
  assert( device() );
  assert( pData );
  assert( nLen >= 0 );

  if ( !nLen )
    return;

  if ( (_nShift & 7) == 0 )     // read aligned !
  {
    if ( _nShift == 0 )
    {
      *(UB*)(pData) = _nByte;
      if ( --nLen  )
        device()->readBlock( pData, nLen );
    }
    else
      device()->readBlock( pData, nLen );
    _nShift = 8;
    return;
  }

  // unaligned read !
  UB *pByte = (UB *)pData;
  while ( nLen-- )
  {
    *pByte = _nByte >> _nShift;
    _nByte = device()->getByte();
    *pByte++ |= _nByte << (8 -_nShift);
  }
}

/** Write a block.
  * @param pData data to write.
  * @param nLen length of data to write in byte.
  */
void EBitStream::writeBlock( const void *pData, SL nLen )
{
  assert( device() );
  assert( pData );
  assert( nLen >= 0 );

  if ( ( _nShift & 7 ) == 0 )   // aligned write !
  {
    writeFlush();
    device()->writeBlock( pData, nLen );
    return;
  }

  // unaligned write !
  const UB *pByte = (const UB *)pData;
  while ( nLen-- )
  {
    _nByte |= *pByte << _nShift;
    device()->putByte( _nByte );
    _nByte = *pByte++ >> (8 -_nShift);
  }
}

/** Get a byte from the stream.
  * @return byte read.
  */
UB EBitStream::getByte()
{
  readFlush();

  UB nVal = _nByte >> _nShift;       // low

  if ( _nShift )                     // hi
  {
    _nByte = device()->getByte();
    nVal |= _nByte << ( 8 -_nShift );
  }
  else
    _nShift = 8;
  return nVal;
}

/** Write a byte in the stream.
  * @param v byte to write.
  */
void EBitStream::putByte( UB v)
{
  writeFlush();

  _nByte |= v << _nShift;

  if ( _nShift )
  {
    device()->putByte( _nByte );
    _nByte = v >> ( 8 -_nShift );
  }
  else
    _nShift = 8;
}

EBitStream& EBitStream::operator <<( const EBitArray& aBits )
{
  assert( device() );
  SL nLen = aBits.count();

  if ( !nLen )
    return *this;

  SL nByteLen = nLen /8;
  const UB *pByte = aBits.datas();
  nLen &= 7;      // final bit shift

  if ( (_nShift & 7) == 0 )   // aligned write !
  {
    writeFlush();
    if ( nByteLen > 0 )
      device()->writeBlock( pByte, nByteLen );
    if ( nLen )
    {
      _nShift = nLen;
      _nByte = pByte[ nByteLen ] & ( (1 << _nShift) -1 );
    }
    return *this;
  }

  // unaligned write !
  while ( nByteLen-- )
  {
    _nByte |= *pByte << _nShift;
    device()->putByte( _nByte );
    _nByte = *pByte++ >> (8 -_nShift);
  }

  if ( nLen )
  {
    UB nByteLeft = *pByte & ( (1 << nLen)-1 );
    _nByte |= nByteLeft << _nShift;
    SL nPosLeft = 8-_nShift;
    if ( nPosLeft < nLen )
    {
      device()->putByte( _nByte );
      _nByte = nByteLeft >> nPosLeft;
      _nShift = nLen - nPosLeft;
    }
    else
      _nShift += nLen;
  }

  return *this;
}






// ########################### EOF EBITSTRM.CPP ##############################
// ########################### EOF EBITSTRM.CPP ##############################
// ########################### EOF EBITSTRM.CPP ##############################

