// ###########################################################################
// File: EBUFFER.CPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################

#include "usuel.hpp"
#include "ebuffer.hpp"

/** Constructs a buffer.
 * Constructs a buffer that can hold at least \a size elements. Set up
 * the binary mode for the EArray using binaryMode.
 * See \l EArray::setBinaryMode( bool ).
 * @param size initial size of the buffer.
 * @see EArray::setBinaryMode( bool ).
 */
EBuffer::EBuffer( int size ) :
    EIODevice()
{
    mBuffer = new TypeArray;
    mOwnBuffer = true;
    mIndex = 0;
}

/** Constructs a buffer.
 * Constructs a buffer that used the dynamic array \a a as buffer.
 * If \a own is true then the array is deleted when the buffer is
 * destroyed.
 * @param size initial size of the buffer.
 * @param own the array is destroyed on destruction is its true.
 */
EBuffer::EBuffer( TypeArray *a, bool own )
{
    mBuffer = a;
    mOwnBuffer = own;
    mIndex = 0;
}

/// Destructor
EBuffer::~EBuffer()
{
  if ( isOpen() )
    close();
  if ( mOwnBuffer )
    delete mBuffer;
}

/** Open the device.
 * Open the device using mode \a _mode.
 * @param _mode mode use to open the device
 */
void EBuffer::open( int _mode )
{
    setMode( _mode );
    setStatus( EIO_IsOpen );
    mIndex = 0;
}

/** Close the device.
 */
void EBuffer::close()
{
  assert( isOpen() );
  if ( isWritable()  &&  ( accessMode() & EIO_Truncate ) )
    mBuffer->resize( mIndex );
  unsetStatus( EIO_IsOpen );
  resetInfo();
}

/** Flush the device.
 */
void EBuffer::flush()
{
    assert( isOpen() );
}

/** Return the size of the device.
 * @return size of the device.
 */
SL EBuffer::size() const
{
    assert( isOpen() );
    return mBuffer->count();
}

/** Return the index of the device.
 * @return index of the device.
 * @exception EXCIODevice on error.
 */
SL EBuffer::at() const
{
    assert( isOpen() );
    return mIndex;
}

/** Set the position of the device.
 * Set the position of the device (index); to \a pos.
 * @param pos new position of the index.
 * @exception EXCIODevice on error.
 */
void EBuffer::at( SL pos)
{
    assert( pos >= 0);

    if ( pos > size() )   // enlarge buffer !
    {
        int oldSize = size();
        mBuffer->resize( pos );
        memset( &mBuffer->at(oldSize), 0, pos -oldSize );
    }
    mIndex = pos;
}

/** Test if the device is at the end.
 * @return true if the device is at the end, false otherwise.
 */
bool EBuffer::atEnd() const
{
    assert( isOpen() );
    return mIndex == size();
}

/** Read a block of byte from the device.
 * Read \a len Byte from the device and store them in the buffer \a data.
 * @param data buffer where data are stored.
 * @param len number of bytes of data to read.
 * @return number of byte of data actually read.
 * @exception EXCIODevice is case of grave error.
 */
SL EBuffer::readBlock( void *data, SL len)
{
    assert( isOpen() );
    assert( isReadable() );

    len = EMIN( size() -mIndex, len );

    if( len == 0 )
      return 0;

    assert( data );

    memcpy( data, &mBuffer->at(mIndex), len );
    mIndex += len;
    return len;
}

/** Write a block of byte on the device.
 * Write \a len Byte of buffer \a data on the device.
 * @param data buffer where data are read.
 * @param len number of bytes of data to write.
 * @exception EXCIODevice is case of grave error.
 */
void 
EBuffer::writeBlock( const void *data, SL len )
{
  assert( len >= 0 );

  if ( len == 0 )
    return;

  assert( len == 0  ||  data );

  if ( mIndex +len > size() )
      mBuffer->resize( mIndex +len );

  memcpy( &mBuffer->at(mIndex), data, len );
  mIndex += len;
}

/** Read a byte from the stream.
 * @return byte read.
 * @exception EIODevice on error
 */
UB EBuffer::getByte()
{
    assert( isOpen() );
    assert( isReadable() );

    if ( atEnd() )
        raiseEXC( EXCIODevice( EXCIODevice::endOfFile  ) );
    return mBuffer->at( mIndex++ );
}

/** Write a Byte on the stream.
 * @param c byte to write.
 * @exception EIODevice on error.
 */
void EBuffer::putByte( UB c )
{
    assert( isOpen() );
    assert( isWritable() );

    if ( atEnd() )
        mBuffer->resize( size() +1 );
    mBuffer->at( mIndex++ ) = c;
}

/** Push-back a byte on the stream.
 * @param c byte to push-back.
 * @exception EIODevice on error.
 */
void EBuffer::ungetByte( UB c )
{
    assert( isOpen() );
    assert( isReadable() );
    mIndex--;
}
