// TfwkArrayT.h: interface for the TfwkArrayT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKARRAYT_H__1D7D9216_4132_11D3_BE28_0040053AD648__INCLUDED_)
#define AFX_TFWKARRAYT_H__1D7D9216_4132_11D3_BE28_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <string.h>
#include "TfwkExc.h"
#include "TfwkArrayItT.h"

/*! \class TfwkArrayT<Obj>
 * \include "TfwkArrayT.h"
 * \brief This template provides a simple array container.
 */
template<class Obj>
class TFWK_UI_EXPORT TfwkArrayT
{
public:
  /*! This array type.
   */
  typedef TfwkArrayT<Obj> TypeArray;

  /*! Type for a const iterator on this array.
   */
  typedef TfwkArrayItT<const TypeArray,const Obj> TfwkConstIt;

  /*! Type for an iterator on this array.
   */
  typedef TfwkArrayItT<TypeArray,Obj> TfwkIt;


  /*! 
   * Constructs the array.
   */
	TfwkArrayT();

  /*!
   * Destructor.
   */
	virtual ~TfwkArrayT();

  /*!
   * Adds an item at the end of the array.
   * @param obj Item to add.
   */
  void add( const Obj &obj );

  /*!
   * Gets the length of the array.
   * @return Length of the array.
   */
  int length() const;

  /*!
   * Gets a reference on a item at a given position in the array.
   * @param nPos Position of the item (zero based).
   * @return Reference on the item at position \a nPos.
   */
  Obj &operator []( int nPos );

  /*!
   * Gets a reference on a item at a given position in the array.
   * @param nPos Position of the item (zero based).
   * @return Reference on the item at position \a nPos.
   */
  Obj &at( int nPos );

  /*!
   * Gets a reference on a item at a given position in the array.
   * @param nPos Position of the item (zero based).
   * @return Reference on the item at position \a nPos.
   */
  const Obj &at( int nPos ) const;

  /*!
   * Finds the a given item.
   * @return Zero based position of the item if found, otherwise -1.
   */
  int find( const Obj &item ) const;

  /*!
   * Removes an item at a given position.
   * @param nPos [In] Position of the item to remove.
   */
  virtual void remove( int nPos );

  /*!
   * Removes all the item and destroyes them.
   */
  virtual void clear();

protected:

  /*!
   * Resizes the array.
   * @param nNewSize New array size.
   */
  void resize( int nNewSize );

private:
  /*!
   * The array.
   */
  Obj *_pData;

  /*!
   * Length of the array.
   */
  int _nLength;
};


// Template implementation : TfwkArrayT<Obj>
// -----------------------------------------

template<class Obj>
TfwkArrayT<Obj>::TfwkArrayT() :
    _pData( NULL ),
    _nLength( 0 )
{
}


template<class Obj>
TfwkArrayT<Obj>::~TfwkArrayT()
{
  if ( _pData )
    free( _pData );
}


template<class Obj>
void 
TfwkArrayT<Obj>::add( const Obj &obj )
{
  resize( _nLength +1 );
  at( _nLength -1 ) = obj;
}


template<class Obj>
inline Obj &
TfwkArrayT<Obj>::operator []( int nPos )
{
  TFWKASSERT( nPos >= 0  &&  nPos < _nLength );
  return _pData[ nPos ];
}


template<class Obj>
inline Obj &
TfwkArrayT<Obj>::at( int nPos )
{
  TFWKASSERT( nPos >= 0  &&  nPos < _nLength );
  return _pData[ nPos ];
}


template<class Obj>
inline const Obj &
TfwkArrayT<Obj>::at( int nPos ) const
{
  TFWKASSERT( nPos >= 0  &&  nPos < _nLength );
  return _pData[ nPos ];
}


template<class Obj>
void 
TfwkArrayT<Obj>::resize( int nNewSize )
{
  TFWKASSERT( nNewSize >=0 );

  if ( !nNewSize )
  {
    free( _pData );
    _pData = NULL;
  }
  else
  {
    Obj *pData = (Obj *)::realloc( _pData, sizeof( Obj ) * nNewSize );
    if ( !pData )   // Out of memory
    {
      TFWKTHROW( TfwkExc::trOutOfMemory );
    }

    _pData = pData;
  }

  _nLength = nNewSize;
}


template<class Obj>
inline int 
TfwkArrayT<Obj>::length() const
{
  return _nLength;
}


template<class Obj>
int 
TfwkArrayT<Obj>::find( const Obj &item ) const
{
  int nLength = length();
  int nPos;

  for ( nPos =0; nPos < nLength; ++nPos )
  {
    if ( at( nPos ) == item )
      return nPos;
  }

  return -1;
}


template<class Obj>
void 
TfwkArrayT<Obj>::remove( int nPos )
{
  if ( nPos < length() )
  {
    memmove( &at( nPos +1), 
             &at(nPos),
             sizeof( Obj ) * ( length() -nPos -1 ) );
    resize( length() -1 );
  }
}


template<class Obj>
void 
TfwkArrayT<Obj>::clear()
{
  resize( 0 );
}


#endif // !defined(AFX_TFWKARRAYT_H__1D7D9216_4132_11D3_BE28_0040053AD648__INCLUDED_)
