// TfwkUnitTest.h: interface for the TfwkUnitTest class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TFWKUNITTEST_H__1D7D9215_4132_11D3_BE28_0040053AD648__INCLUDED_)
#define AFX_TFWKUNITTEST_H__1D7D9215_4132_11D3_BE28_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TfwkClass.h"
#include "TfwkUnitTestMethodInfo.h"
#include "TfwkUnitTestInfoT.h"

/*!
 * This macro is used to make an assertion in the unit test.
 *
 * Throw exception TfwkExc with reason set to TfwkExc::trAssertionFailed
 * if the assertion failed.
 */
#define tassert( bCondition )                                     \
  {                                                               \
    if ( !(bCondition) )                                          \
    {                                                             \
      TFWKTHROWINFO( TfwkExc::trAssertionFailed, #bCondition );   \
    }                                                             \
  }


/*!
 * This macro must be used to declare a class as a unit test.
 *
 * Access is set to \c public after the macro.
 *
 * @param Class Class to declare as a unit test.
 * 
 * Example:
 *
 * \code
 * class MyUnitTest : public TfwkUnitTest
 * {
 * public:
 *   TFWKDECL_UNITTEST( MyUniTest );
 *   void testAdd();
 *   void testSub();
 *   void testMul();
 * };
 * \encode
 *
 * \sa TFWKREG_START, TFWKREG, TFWKREG_END, TfwkUnitTest.
 */
#define TFWKDECL_UNITTEST( Class )                                     \
    public:                                                            \
      typedef TfwkUnitTestInfoT<Class> TypeTestInfo;                   \
      TFWKDECL_DYNCLASS;                                               \
      static TfwkUnitTestMethodInfo _infoUnitTestMethod[];             \
    private:                                                           \
      static TypeTestInfo _infoUnitTest;                               \
    public:


/*!
 * This macro is used to start declaring unit test method.
 * 
 * It must be use in the implementation (.cpp) file.
 *
 * @param Class Class declared as a unit test.
 * @param ParentClass Parent class of \a Class (usually TfwkUnitTest).
 * 
 * Example:
 * 
 * \code
 * TFWKREG_START( MyUnitTest, TfwkUnitTest )
 *   TFWKREG( testAdd )
 *   TFWKREG( testSub )
 *   TFWKREG( testMul )
 * TFWKREG_END();
 * \encode
 *
 * \sa TFWKDECL_UNITTEST, TFWKREG, TFWKREG_END, TfwkUnitTest.
 */
#define TFWKREG_START( Class, ParentClass )                                   \
    TFWKIMPL_DYNCLASS( Class, ParentClass );                                  \
                                                                              \
    Class::TypeTestInfo Class::_infoUnitTest(  TFWKCLASSID( Class),           \
                                               Class::_infoUnitTestMethod );  \
                                                                              \
    TfwkUnitTestMethodInfo Class::_infoUnitTestMethod[] =                     \
    {

 
/*!
 * This macro is used to declare a unit test method.
 *
 * @param method The name of the method that implement the test. The method
 * should return \c void and take no argument. 
 *
 * The prototype of a test method is as follow : \c void \c testSomething();
 * 
 * See TFWKREG_START for an example of use.
 *
 * \sa TFWKDECL_UNITTEST, TFWKREG_START, TFWKREG_END, TfwkUnitTest.
 */
#define TFWKREG( method ) \
      TfwkUnitTestMethodInfo(#method, (TfwkUnitTestMethodInfo::TypeMethod)method ),


 /*!
 * This macro ends the declaration of a unit test.
 *
 * See TFWKREG_START for an example of use.
 *
 * \sa TFWKDECL_UNITTEST, TFWKREG_START, TFWKREG, TfwkUnitTest.
 */
#define TFWKREG_END()             \
      TfwkUnitTestMethodInfo()    \
    }




/*! \class TfwkUnitTest
 * \include "TfwkUnitTest.h"
 * \brief Base class for all unit test.
 *
 *
 * \sa TFWKDECL_UNITTEST, TFWKREG_START, TFWKREG, TFWKREG_END.
 */
class TFWK_UI_EXPORT TfwkUnitTest : public TfwkClass
{
public:
  TFWKDECL_DYNCLASS;

  /*!
   * Constructs a Unit Test.
   */
  TfwkUnitTest() {}

  /*!
   * Destructor.
   */
  virtual ~TfwkUnitTest() {}

  /*!
   * Stupid VC5 does not compile with a 't allow default constructor for 
   * TfwkUnitTestMethodInfoT<UnitTest>, so this is hack.
   */
  void tfwkDummyMethod();
};



#endif // !defined(AFX_TFWKUNITTEST_H__1D7D9215_4132_11D3_BE28_0040053AD648__INCLUDED_)
